/*
 * config.c, part of e
 * read COPYING now for more inportant info.
 *
 * configuration-issues
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "e.h"


int config_readkeys( void );
int config_readeditor( void );
int config_getvalue( char *, int * );
int config_getstring( char *, char ** );


struct str_keydef {
	char	*name;
	int	value;
};

FILE			*cfgfile;
struct str_keydef	keydef[] = {
			{ "insert",	331 },
			{ "home",	262 },
			{ "pageup",	339 },
			{ "delete",	330 },
			{ "end",	360 },
			{ "pagedown",	338 },
			{ "down",	258 },
			{ "up",		259 },
			{ "left",	260 },
			{ "right",	261 },
			{ "num0",	331 },
			{ "num1",	360 },
			{ "num2",	258 },
			{ "num3",	338 },
			{ "num4",	260 },
			{ "num5",	350 },
			{ "num6",	261 },
			{ "num7",	262 },
			{ "num8",	259 },
			{ "num9",	339 },
			{ "backspace",	263 },
			{ NULL,		0 }
			};


/*
 * tries to interpret the "~/.erc" file
 *
 * returns 0 on error, 1 on success
 */
int config_readconfig( void )
{
	char		configline[256];
	char		*tmp;
	char		cfgfilename[512];
	
	snprintf( cfgfilename, 512, "%s/.erc", e->homepath );
	
	e_log( "config started" );
	
	if( !( cfgfile = fopen( cfgfilename, "r" ) ) ) {
		printf( "Couldn't open config-file '%s'!\n", cfgfilename );
		return 0;
	}

	while( !feof( cfgfile ) ) {
		fgets( configline, 256, cfgfile );
		tmp = seekchar( configline, '[' );
		if( !strncasecmp( tmp, "[keys]", 6 ) ) {
			if( !config_readkeys() ) return 0;
		} else if( !strncasecmp( tmp, "[editor]", 8 ) ) {
			if( !config_readeditor() ) return 0;
		}
	}
	
	e_log( "config stopped\n" );
	
	fclose( cfgfile );
	return 1;
}


/*
 * tries to read the [keys]-section
 *
 * returns 0 on error, 1 on success
 */
int config_readkeys( void )
{
	char		configline[256];
	char		*tmp;
	int		shift, ctrl, meta;
	int		keyvalue[MAX_KEYS + 1];
	char		*macro;
	int		i, tmpi, tmpi2, nesting;
	STR_KEY		*newkey;
	char		keyname[128];
	int		filepos;

	e_log( "  keyconfig started" );

	while( !feof( cfgfile ) ) {
		filepos = ftell( cfgfile );
		if( !fgets( configline, 256, cfgfile ) ) break;
		toeol( configline, configline, 256 );
		tmp = stripspaces( configline );
		if( tmp[0] == '#' )
			continue;
		if( tmp[0] == '[' ) {
			fseek( cfgfile, filepos, SEEK_SET );
			break;
		}
		if( !tmp[0] || tmp[0] == '\n' ) continue;
		
		shift = 0; ctrl = 0; meta = 0; keyname[0] = 0;
		for( i = 0; i < MAX_KEYS + 1; i++ )
			keyvalue[i] = 0;
		if( !strncasecmp( tmp, "shift_", 6 ) ) {
			shift = 1;
			strncat( keyname, tmp, 6 );
			tmp += 6;
		}
		if( !strncasecmp( tmp, "ctrl_", 5 ) ) {
			ctrl = 1;
			strncat( keyname, tmp, 5 );
			tmp += 5;
		}
		if( !strncasecmp( tmp, "meta_", 5 ) ) {
			keyvalue[0] = 27;
			meta = 1;
			strncat( keyname, tmp, 5 );
			tmp += 5;
		}
		for( i = 0; keydef[i].name; i++ )
			if( !strncasecmp( tmp, keydef[i].name, strlen( keydef[i].name ) ) ) break;
		if( keydef[i].name ) {
			keyvalue[0] = keydef[i].value;
			if( keyvalue[0] >= 265 && keyvalue[0] <= 274 ) 
				if( shift ) keyvalue[0] += 10;
			strcat( keyname, keydef[i].name );
			tmp += strlen( keydef[i].name );
		} else {
			for( i = meta; tmp[0]; i++ ) {
				keyvalue[i] = tmp[0];
				if( isalpha( keyvalue[i] ) ) {
					keyvalue[i] = toupper( keyvalue[i] );
					if( ctrl ) keyvalue[i] -= 64;
				}
				strncat( keyname, tmp, 1 );
				tmp += 2;
				if( tmp[-1] != '_' ) break;
				strncat( keyname, tmp - 1, 1 );
				if( i >= MAX_KEYS + 1 ) {
					e_msg( "e: invalid configline: %s", configline );
					return 0;
				}
			}
		}
		if( isalpha( tmp[0] ) || tmp[0] == '_' ) {
			e_msg( "e: invalid configline: %s", configline );
			return 0;
		}
		tmp = seekchar( tmp, '=' );
		if( tmp[0] != '=' ) {
			e_msg( "e: invalid configline: %s", configline );
			return 0;
		} else tmp++;
		tmp = stripspaces( tmp );
		if( tmp[0] == '{' ) {
			nesting = 1;
			tmpi = ftell( cfgfile );
			while( !feof( cfgfile ) ) {
				i = fgetc( cfgfile );
				if( i == '{' ) nesting++;
				else if( i == '}' ) nesting--;
				if( !nesting ) break;
			}
			tmpi2 = ftell( cfgfile ) - tmpi;
			if( !( macro = (char *) e_alloc( tmpi2 ) ) ) return 0;
			fseek( cfgfile, tmpi, SEEK_SET );
			fread( macro, 1, tmpi2, cfgfile );
		} else macro = strdup( tmp );

		if( !( newkey = (STR_KEY *) e_alloc( sizeof( STR_KEY ) ) ) ) return 0;

		if( e->keys ) {
			if( e->keys->next ) {
				newkey->next = e->keys->next;
				newkey->next->prev = newkey;
			} else newkey->next = NULL;
			newkey->prev = e->keys;
			e->keys->next = newkey;
		} else {
			e->keys = newkey;
			newkey->next = NULL;
			newkey->prev = NULL;
		}
		newkey->macro = macro;
		strncat( newkey->name, keyname, 128 );
		for( i = 0; i < MAX_KEYS + 1; i++ )
			newkey->value[i] = keyvalue[i];
		
		e_log( "    %s = %s", newkey->name, newkey->macro );
	}
	
	e_log( "  keyconfig stopped" );
	
	return 1;
}


/*
 * tries to read the [editor]-section
 *
 * returns 0 on error, 1 on success
 */
int config_readeditor( void )
{
	char		configline[256];
	char		*tmp, *s;
	int		value;

	e_log( "  editorconfig started" );
	
	/* defaults */
	e->tabsize = 8;
	e->statusbarpos = "+0";
	e->statusbarpriority = 0;
	e->statusbarleft = "\\p";
	e->statusbarright = "\\m  Line \\l   Col \\c  \\T  ^KH help";

	while( !feof( cfgfile ) ) {
		if( !fgets( configline, 256, cfgfile ) ) break;
		toeol( configline, configline, 256 );
		tmp = stripspaces( configline );
		if( tmp[0] == '#' )
			continue;
		if( tmp[0] == '[' )
			return 0;
		if( !tmp[0] || tmp[0] == '\n' ) continue;
				
		if( !strncasecmp( tmp, "tabsize", 7 ) ) {
			if( !config_getvalue( tmp, &value ) )
				return 0;
			e->tabsize = value;
			e_log( "    tabsize = %d", e->tabsize );
		} else if( !strncasecmp( tmp, "statusbarpos", 12 ) ) {
			if( !config_getstring( tmp, &s ) )
				return 0;
			e->statusbarpos = s;
			e_log( "    statusbarpos = %s", e->statusbarpos );
		} else if( !strncasecmp( tmp, "statusbarpriority", 17 ) ) {
			if( !config_getvalue( tmp, &value ) )
				return 0;
			e->statusbarpriority = value;
			e_log( "    statusbarpriority = %d", e->statusbarpriority );
		} else if( !strncasecmp( tmp, "statusbarleft", 13 ) ) {
			if( !config_getstring( tmp, &s ) )
				return 0;
			e->statusbarleft = s;
			e_log( "    statusbarleft = %s", e->statusbarleft );
		} else if( !strncasecmp( tmp, "statusbarright", 14 ) ) {
			if( !config_getstring( tmp, &s ) )
				return 0;
			e->statusbarright = s;
			e_log( "    statusbarright = %s", e->statusbarright );
		}
	}
	
	e_log( "  editorconfig stopped" );
	
	return 1;
}


/*
 * reads the righthand side of a config-string and interprets it as a value
 *
 * returns: 0 on error, 1 on success
 */
int config_getvalue( char *s, int *value )
{
	char		*s2;
	
	s2 = seekchar( s, '=' );
	if( s2[0] != '=' ) {
		e_msg( "e: invalid configline: %s", s );
		return 0;
	} else
		s2++;
	s2 = stripspaces( s2 );
	
	*value = atoi( s2 );
	
	return 1;
}


/*
 * reads the righthand side of a config-string and interprets it as a string
 *
 * returns: 0 on error, 1 on success ( s2 = ptr to allocated string )
 */
int config_getstring( char *s, char **s2 )
{
	char		*tmp;
	
	tmp = seekchar( s, '=' );
	if( tmp[0] != '=' ) {
		e_msg( "e: invalid configline: %s", s );
		return 0;
	} else
		tmp++;
	tmp = stripspaces( tmp );
	*s2 = strdup( tmp );
	return 1;
}
