/*
 * e v0.2.1 (C) 1997 Mikael Klasson aka Fluff of Logic Design
 * read COPYING now for more important info.
 *
 * Errorlevels:	0 	normal
 *		1	couldn't open file
 *		2	not enough memory
 *		3	error during config (whatever, including lack of mem)
 *		4	couldn't open logfile
 *		5	unknown option on commandline
 */


#include <stdarg.h>
#include <stdio.h>
#include <curses.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include <string.h>
#include "e.h"


#define VERSION "0.2.1"


STR_KEY *e_getkey( STR_KEY *, int, int * );
void e_atexit( void );


STR_E		*e;
int		nokeymatch;


int main( int argc, char **argv )
{
	int		i, i2;
	int		error = 0;
	int		key[MAX_KEYS + 1];
	int		mode = 0; /* 0	normal; just filenames on cmdline
		 	 	   * 1	help
		 	 	   * 2	-- spotted; all filenames from now on
		 	 	   */
	char		logfilename[256];
	STR_KEY		*k;

	if( argc < 2 ) {
		printf( "e: usage: e [OPTIONS] [--] FILENAME[S]\n" );
		printf( "for more help, run: e -h\n" );
		return 0;
	}

	if( !( e = (STR_E *) e_alloc( sizeof( STR_E ) ) ) ) return 2;

	e->homepath = getenv( "HOME" );
	e->keys = NULL;
	e->curwin = NULL;
	e->logging = 0;
	e->status = 0;

	for( i = 1; i < argc; i++ ) {
		if( argv[i][0] == '-' ) {
			if( !strncmp( argv[i], "--", 2 ) ) {
				if( !strcmp( argv[i], "--help" ) ) mode = 1;
				else if( !strcmp( argv[i], "--log" ) ) e->logging = 1;
				else if( !strcmp( argv[i], "--printconfig" ) ) mode = 3;
				else if( !strcmp( argv[i], "--" ) ) break;
				else {
					printf( "e: unknown option '%s'\n", argv[i] );
					error = 1;
				}
			} else for( i2 = 1; i2 < strlen( argv[i] ); i2++ ) {
				if( argv[i][i2] == 'h' ) mode = 1;
				else if( argv[i][i2] == 'l' ) e->logging = 1;
				else if( argv[i][i2] == 'p' ) mode = 3;
				else {
					printf( "e: unknown option '-%c'\n", argv[i][i2] );
					error = 1;
				}
			}
		}
	}
	
	if( error ) exit( 5 );
	
	if( mode == 1 ) {
		printf( "\ne v%s (C) 1997 Mikael Klasson aka Fluff of Logic Design\n", VERSION );
		printf( "usage: e [OPTIONS] [--] FILENAME[S]\n\n" );
		printf( "  -h, --help		show this help\n" );
		printf( "  -l, --log		log all actions to ~/.e_log\n" );
		printf( "  -p, --printconfig	display configuration info (keymappings etc)\n\n" );
		printf( "everything after -- is interpreted as filenames\n\n" );
		return 0;
	}

	if( e->logging ) {
		snprintf( logfilename, 256, "%s/.e_log", e->homepath );
		if( !( e->logfile = fopen( logfilename, "w" ) ) ) {
			printf( "e: unable to open logfile '%s' for writing!\n", logfilename );
			return 4;
		}
		e_log( "starting up, truncating logfile\n" );
	}

	if( !config_readconfig() ) {
		printf( "e: error during config!\n" );
		macro_run( "exit 3" );
	}

	if( mode == 3 ) {
		printf( "[keys]\n" );
		for( k = e->keys; k; k = k->next )
			printf( "%-15s = %s\n", k->name, k->macro );
		printf( "\n[editor]\n" );
		printf( "tabsize \t= %d\n", e->tabsize );
		macro_run( "exit 0" );
	}

	initscr();
	cbreak();
//	raw();
	noecho();
	keypad( stdscr, TRUE );
	scrollok( stdscr, TRUE );
	atexit( (void *) endwin );
	atexit( (void *) e_atexit );
	refresh();

	e->status = 1;

	if( !macro_run( "newwin" ) ) macro_run( "exit 2" );

	for( i = 1; i < argc; i++ ) {
		if( !strcmp( argv[i], "--" ) && mode != 2 ) {
			mode = 2;
		} else if( argv[i][0] != '-' || mode == 2 ) {
			if( !macro_run( "loadfile %s", argv[i] ) ) {
				e_msg( "couldn't load file '%s'!", argv[i] );
				getch();
			}
		} 
	}

	refresh();

	if( !e->curwin->file ) {
		printf( "e: no files loaded, exiting!" );
		macro_run( "exit 1" );
	}
	
	while( TRUE ) {
		e_getkey( e->keys, 0, key );
	}

	return 0;
}


STR_KEY *e_getkey( STR_KEY *k, int nesting, int *key )
{
	int		i;
	char		buf[MAX_KEYS + 1];

	if( !nesting ) nokeymatch = 1;
	key[nesting] = getch();
	e_msg( "" );
	key[nesting + 1] = 0;
	for( ; k && nokeymatch; ) {
		for( i = 0; i <= nesting; i++ ) {
			if( isalpha( key[i] ) ) {
				if( k->value[i] != toupper( key[i] ) ) break;
			} else if( k->value[i] != key[i] ) break;
		}
		if( i > nesting ) {
			if( k->value[nesting + 1] == 0 || nesting >= MAX_KEYS - 1 ) {
				macro_run( k->macro );
				nokeymatch = 0;
			} else k = e_getkey( k, nesting + 1, key );
		}
		if( k ) k = k->next;
	}
	if( !nesting && nokeymatch ) {
		for( i = 0; key[i]; i++ )
			buf[i] = key[i];
		buf[i] = 0;
		do_insert( buf );
	}
	return k;
}


void e_atexit( void )
{
	if( e->logging ) fclose( e->logfile );
}


/*
 * tries to allocate mem
 *
 * returns: NULL on error, (void *) on success
 */
void *e_alloc( int size )
{
	void 		*p;

	if( !( p = malloc( size ) ) ) {
		printf( "e: not enough mem!\n" );
		return NULL;
	}
	
	return p;
}


void e_log( const char *format, ... )
{
	struct tm 	*t;
	long		timet;
	va_list		ap;

	if( !e->logging ) return;

	time( &timet );
	t = localtime( &timet );
	
	va_start( ap, format );
	
	fprintf( e->logfile, "[%.4d-%.2d-%.2d %.2d.%.2d.%.2d] ", t->tm_year + 1900, t->tm_mon+1, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec );
	vfprintf( e->logfile, format, ap );
	fputc( '\n', e->logfile );
}


void e_msg( const char *format, ... )
{
	char		msgbuf[MAX_STRINGSIZE];
	va_list		ap;
	
	va_start( ap, format );
	
	vsnprintf( msgbuf, MAX_STRINGSIZE, format, ap );
	do_showstatus( msgbuf );
}
