
#ifndef __e_h

#define __e_h


#include <stdio.h>
#include <stdarg.h>


/*************************************** definitions */

#define	MAX_KEYS	3	/* maximum number of keycodes you're allowed
				   to have in your keymappings. Note that
				   a META-key adds a 27-prefix, so that uses 
				   up one value... */

#define MAX_LINESIZE	1024	/* maximum line-length. I cant see any
				   practical reason to why you'd want more,
				   but here it is, just in case... */

#define MAX_STRINGSIZE	512	/* used when parsing va_list into a regular
				   string. No need to fiddle with this one,
				   it's only used internally and should be
				   large enough unless I've erred... */

#define ALLOC_SIZE	40	/* increasing this would waste more memory,
				   but could be useful if you experience lots
				   of memory-fragmentation */

#define MAX_PATHSIZE	512	/* maximum length of a full pathname, including
				   filename */

#define MAX_STATUSBARSIZE 256	/* maximum length of a parsed statusbar
				   definition */


/*************************************** structures */

typedef struct str_file		STR_FILE;
typedef struct str_key		STR_KEY;
typedef struct str_e		STR_E;
typedef struct str_win		STR_WIN;
typedef struct str_line		STR_LINE;


struct str_file {
	STR_LINE	*firstline;	/* ptr to first STR_LINE */
	STR_LINE	*curline;	/* ptr to current STR_LINE */
	int		line;		/* line in file */
	int		col;		/* column in file */
	int		topline;	/* linenumber of topmost line */
	int		leftcol;	/* colnumber of leftmost column */
	FILE		*orig;		/* original file */
	char		name[MAX_PATHSIZE];	/* filename */
	int		size;		/* filelength */
	int		tabsize;	/* tabsize */
	STR_FILE	*prev;		/* ptr to prev file loaded */
	STR_FILE	*next;		/* ptr to next file loaded */
};


struct str_line {
	int		bufpos;		/* 0 == in mem
					   1 == in original file
					   2 == in swapfile */
	int		offset;		/* offset of datablock in file */
	int		size;		/* length of buf in bytes */
	int		maxsize;	/* size of allocated block */
	char		*buf;		/* ptr to datablock */
	STR_LINE	*prev;		/* ptr to prev STR_LINE */
	STR_LINE	*next;		/* ptr to next STR_LINE */
};


struct str_key {
	char		name[128];
	int		value[MAX_KEYS + 1];	/* always zero-padded */
	char		*macro;
	STR_KEY		*prev;
	STR_KEY		*next;
};


struct str_e {
	STR_WIN		*curwin;
	char		*homepath;
	STR_KEY		*keys;
	FILE		*logfile;
	int		logging;
	int		status;		/* 0 == not inside editor, 1 == inside */
	int		tabsize;	/* default tabsize, filespecific tabsizes override this */
	char		*statusbarpos;	/* position of statusbar */
	int		statusbarpriority;	/* 0 = right, 1 = left */
	char		*statusbarleft;	/* ptr to definition for left side of
					   statusbar */
	char		*statusbarright;
};


struct str_win {
	STR_FILE	*file;
	STR_LINE	*curline;	/* ptr to current STR_LINE */
	int		line;		/* line in file */
	int		col;		/* column in file */
	int		topline;	/* linenumber of topmost line */
	int		leftcol;	/* colnumber of leftmost column */

	int		xpos;		/* x-coord of window */
	int		ypos;		/* y-coord of window */
	int		width;		/* width of window */
	int		height;		/* height of window */

	STR_WIN		*prev;		/* ptr to prev window */
	STR_WIN		*next;		/* ptr to next window */
};


/*************************************** variables */

extern STR_E	*e;


/*************************************** functions */


/* e.c ********************************/

extern void	*e_alloc( int );
extern void	e_log( const char *, ... );
extern void	e_msg( const char *, ... );


/* macro.c ****************************/

extern int	macro_run( const char *, ... );

extern int	do_exit( char * );

extern int	do_refresh( char * );
extern int	do_refreshline( char * );
extern int	do_refreshcursor( char * );
extern int	do_refreshstatusbar( char * );

extern int	do_showstatus( char * );

extern int	do_delline( char * );
extern int	do_delleft( char * );
extern int	do_delright( char * );

extern int	do_loadfile( char * );
extern int	do_closefile( char * );
extern int	do_savefile( char * );

extern int	do_nextfile( char * );
extern int	do_prevfile( char * );

extern int	do_newwin( char * );
extern int	do_closewin( char * );

extern int	do_up( char * );
extern int	do_down( char * );
extern int	do_left( char * );
extern int	do_right( char * );
extern int	do_startofline( char * );
extern int	do_endofline( char * );
extern int	do_pageup( char * );
extern int	do_pagedown( char * );

extern int	do_cursoroffset( char * );

extern int	do_insert( char * );


/* config.c ***************************/

extern int	config_readconfig( void );


/* misc.c *****************************/

extern char	*seekalpha( char * );
extern char	*seekchar( char *, char );
extern char	*seeknextline( char * );
extern char	*stripspaces( char * );
extern void	toeol( char *, char *, int );

extern int	readlinetomem( void );


/*****************************************************************************/

#endif
